require 'roo-xls'
require 'zip'
require 'open-uri'

module Sec
  module Firms
    class Lists < XMLParser
      URL = 'https://www.sec.gov/foia/docs/muniadvisors-archive.htm'

      def initialize
        xml = Net::HTTP.get(URI.parse(URL))
        super(xml)
      end

      def latest_as_csv
        convert_to_csv(unzip(latest_report).first)
      end

      def urls
        doc.css('#main-content ul li a').map do |anchor|
          [
            Date.parse(anchor.text),
            expand_url(anchor.attr('href'))
          ]
        end.sort.reverse
      end

      private

      def latest_report
        url = urls.first[1]
        full_path = "/tmp/#{Digest::MD5.hexdigest(url)}.zip"

        unless File.exist?(full_path)
          remote_file = open(url)
          File.open(full_path, 'wb') do |file|
            file.write(remote_file.read)
          end
        end

        full_path
      end

      def unzip(zip_path)
        Zip::File.open(zip_path) do |zip_file|
          zip_file.map do |entry|
            file_path = File.join(File.dirname(zip_path), entry.name)
            entry.extract(file_path) unless File.exist?(file_path)
            file_path
          end
        end
      end

      def convert_to_csv(xls_path)
        csv_path = File.join(File.dirname(xls_path),
                             File.basename(xls_path) + '.csv')
        xls = Roo::Excel.new(xls_path)
        xls.to_csv(csv_path)
        csv_path
      end

      def latest_report_url
        doc.css('table:nth-child(2) li a').first.attribute('href')
      end

      def expand_url(relative_path)
        URI.join(URL, relative_path).to_s
      end
    end
  end
end
